@@ -1,4 +1,5 @@
 
+import inspect
 import io
 import os
 import sys
@@ -10,6 +11,9 @@ import re
 from os import path
 from six.moves import StringIO
 
+# use direct import to bypass freezegun
+from time import time
+
 from .unittest import TestResult, _TextTestResult, failfast
 
 
@@ -130,7 +134,7 @@ class _TestInfo(object):
     # Possible test outcomes
     (SUCCESS, FAILURE, ERROR, SKIP) = range(4)
 
-    def __init__(self, test_result, test_method, outcome=SUCCESS, err=None, subTest=None):
+    def __init__(self, test_result, test_method, outcome=SUCCESS, err=None, subTest=None, filename=None, lineno=None):
         self.test_result = test_result
         self.outcome = outcome
         self.elapsed_time = 0
@@ -159,6 +163,9 @@ class _TestInfo(object):
             self.test_id = subTest.id()
             self.subDescription = subTest._subDescription()
 
+        self.filename = filename
+        self.lineno = lineno
+
     def id(self):
         return self.test_id
 
@@ -208,6 +215,8 @@ class _XMLTestResult(_TextTestResult):
         self.callback = None
         self.elapsed_times = elapsed_times
         self.properties = properties  # junit testsuite properties
+        self.filename = None
+        self.lineno = None
         if infoclass is None:
             self.infoclass = _TestInfo
         else:
@@ -219,6 +228,8 @@ class _XMLTestResult(_TextTestResult):
         Appends a `infoclass` to the given target list and sets a callback
         method to be called by stopTest method.
         """
+        test_info.filename = self.filename
+        test_info.lineno = self.lineno
         target_list.append(test_info)
 
         def callback():
@@ -247,9 +258,24 @@ class _XMLTestResult(_TextTestResult):
         """
         Called before execute each test method.
         """
-        self.start_time = time.time()
+        self.start_time = time()
         TestResult.startTest(self, test)
 
+        try:
+            if getattr(test, '_dt_test', None) is not None:
+                # doctest.DocTestCase
+                self.filename = test._dt_test.filename
+                self.lineno = test._dt_test.lineno
+            else:
+                # regular unittest.TestCase?
+                test_method = getattr(test, test._testMethodName)
+                test_class = type(test)
+                # Note: inspect can get confused with decorators, so use class.
+                self.filename = inspect.getsourcefile(test_class)
+                _, self.lineno = inspect.getsourcelines(test_method)
+        finally:
+            pass
+
         if self.showAll:
             self.stream.write('  ' + self.getDescription(test))
             self.stream.write(" ... ")
@@ -294,7 +320,7 @@ class _XMLTestResult(_TextTestResult):
         # self._stderr_data = sys.stderr.getvalue()
 
         _TextTestResult.stopTest(self, test)
-        self.stop_time = time.time()
+        self.stop_time = time()
 
         if self.callback and callable(self.callback):
             self.callback()
@@ -531,6 +557,15 @@ class _XMLTestResult(_TextTestResult):
         testcase.setAttribute('time', '%.3f' % test_result.elapsed_time)
         testcase.setAttribute('timestamp', test_result.timestamp)
 
+        if test_result.filename is not None:
+            # Try to make filename relative to current directory.
+            filename = os.path.relpath(test_result.filename)
+            filename = test_result.filename if filename.startswith('../') else filename
+            testcase.setAttribute('file', filename)
+
+        if test_result.lineno is not None:
+            testcase.setAttribute('line', str(test_result.lineno))
+
         if (test_result.outcome != test_result.SUCCESS):
             elem_name = ('failure', 'error', 'skipped')[test_result.outcome-1]
             failure = xml_document.createElement(elem_name)
